(define-module (glicid services openldap)
  #:use-module (glicid packages openldap)
  #:use-module (gnu services)
  #:use-module (gnu services shepherd)
  #:use-module (guix)
  #:use-module (guix records)
  #:use-module (ice-9 match)
  #: export (
    openldap-configuration
    openldap-configuration?
    %default-slapd.conf
    openldap-shepherd-service
    openldap-service-type
  )
)

(define-record-type* <openldap-configuration>
  openldap-configuration make-openldap-configuration
  openldap-configuration?
  (openldap openldap-configuration-openldap
    (default openldap)
  )
  (uri openldap-configuration-uri
    (default "ldapi:// ldap://")
  )
  (logflags openldap-configuration-logflags
    (default "0")
  )
  (pid-file openldap-configuration-pid-file
    (default "/var/run/openldap/slapd.pid")
  )
  (config-file openldap-configuration-config-file
    (default %default-slapd.conf)
  )
  (log-file openldap-configuration-log-file
    (default "/var/log/slapd.log")
  )
)

(define %default-slapd.conf
  (plain-file "slapd.conf" "
   # Empty file for test
"))

(define openldap-shepherd-service
  (match-lambda
    (($ <openldap-configuration> openldap uri logflags pid-file config-file log-file)
      (list
        (shepherd-service
          (provision '(slapd) )
          (documentation "Run openldap.")
          (requirement '(user-processes))
          (respawn? #f)
          (start #~(make-forkexec-constructor
            (list
              #$(file-append openldap "/libexec/slapd")
              "-h" #$uri
              "-d" #$logflags
              "-f" #$config-file
            )
            #:pid-file #$pid-file 
            #:log-file #$log-file
          ))
          (stop #~(make-kill-destructor))
        )
      )
    )
  )
)

(define openldap-service-type
  (service-type (name 'slapd)
    (extensions 
      (list (
        service-extension
        shepherd-root-service-type
        openldap-shepherd-service
      ))
    )
    (description "Run @uref{https://www.openldap.org, Openldap} community developped LDAP software.")
  )
)
